@@ -3,7 +3,7 @@
  * espeak.c - Speech Dispatcher backend for espeak
  *
  * Copyright (C) 2007 Brailcom, o.p.s.
- * Copyright (C) 2019-2021 Samuel Thibault <samuel.thibault@ens-lyon.org>
+ * Copyright (C) 2019-2022 Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -223,6 +223,10 @@ int module_init(char **status_info)
 		DBG(DBG_MODNAME " Failed to set punctuation list.");
 
 	espeak_voice_list = espeak_list_synthesis_voices();
+	if (espeak_voice_list == NULL) {
+		*status_info = g_strdup(DBG_MODNAME " has no voice.");
+		return FATAL_ERROR;
+	}
 
 	initialized = TRUE;
 	*status_info = g_strdup(DBG_MODNAME " Initialized successfully.");
@@ -247,22 +251,21 @@ void module_speak_sync(const gchar * data, size_t bytes, SPDMessageType msgtype)
 	    (unsigned long)bytes);
 
 	/* Setting speech parameters. */
-	UPDATE_STRING_PARAMETER(voice.language, espeak_set_language);
-	UPDATE_PARAMETER(voice_type, espeak_set_voice);
-	UPDATE_STRING_PARAMETER(voice.name, espeak_set_synthesis_voice);
-
-	if (msg_settings_old.voice.name && strstr(msg_settings_old.voice.name , "mbrola"))
+#ifdef ESPEAK_NG_INCLUDE
+	if (EspeakMbrola)
 	{
-		if (msgtype == SPD_MSGTYPE_TEXT)
-		{
-			/* espeak has troubles with setting mbrola voices in SSML mode
-			 * https://github.com/espeak-ng/espeak-ng/issues/1011 */
-			DBG(DBG_MODNAME " mbrola voice, stripping SSML.");
-			msg = module_strip_ssml(data);
-			data = msg;
-		}
-
-		flags &= ~espeakSSML;
+		/* espeak has troubles with setting mbrola voices in SSML mode
+		 * https://github.com/espeak-ng/espeak-ng/issues/1011 */
+		espeak_set_language(msg_settings.voice.language);
+		espeak_set_voice(msg_settings.voice_type);
+		espeak_set_synthesis_voice(msg_settings.voice.name);
+	}
+	else
+#endif
+	{
+		UPDATE_STRING_PARAMETER(voice.language, espeak_set_language);
+		UPDATE_PARAMETER(voice_type, espeak_set_voice);
+		UPDATE_STRING_PARAMETER(voice.name, espeak_set_synthesis_voice);
 	}
 
 	UPDATE_PARAMETER(rate, espeak_set_rate);
@@ -842,6 +845,14 @@ static SPDVoice **espeak_list_synthesis_voices()
 
 	numvoices = g_queue_get_length(voice_list);
 	DBG(DBG_MODNAME " %d voices total.", numvoices);
+#ifdef ESPEAK_NG_INCLUDE
+	if (!EspeakMbrola)
+#endif
+	{
+		if (numvoices == 0) {
+			return NULL;
+		}
+	}
 
 	memset(&voice_spec, 0, sizeof(voice_spec));
 	voice_spec.languages = "variant";
@@ -880,7 +891,6 @@ static SPDVoice **espeak_list_synthesis_voices()
 		{
 			const char *identifier = espeak_mbrola[j]->identifier;
 			char *voicename, *dash, *path;
-			struct stat st;
 
 			totnummbrola++;
 
@@ -930,6 +940,15 @@ static SPDVoice **espeak_list_synthesis_voices()
 #endif
 
 	totalvoices = (numvoices * (numvariants + 1)) + nummbrola;
+
+	if (totalvoices == 0) {
+		if (voice_list != NULL)
+			g_queue_free(voice_list);
+		if (variant_list != NULL)
+			g_queue_free_full(variant_list, (GDestroyNotify)g_free);
+		return NULL;
+	}
+
 	result = g_new0(SPDVoice *, totalvoices + 1);
 	voice_list_iter = g_queue_peek_head_link(voice_list);
 
